@@ -76,6 +76,14 @@ POOL_MAX_CONN = 25
 log = logging.getLogger("rawl")
 
 
+def pop_or_none(d, k):
+    """ Pop a value from a dict or return None if not exists """
+    try:
+        return d.pop(k)
+    except KeyError:
+        return None
+
+
 class RawlException(Exception):
     pass
 
@@ -103,7 +111,9 @@ class RawlConnection(object):
 
         # Create the pool if it doesn't exist already
         if RawlConnection.pool is None:
-            RawlConnection.pool = ThreadedConnectionPool(POOL_MIN_CONN, POOL_MAX_CONN, self.dsn)
+            RawlConnection.pool = ThreadedConnectionPool(
+                POOL_MIN_CONN, POOL_MAX_CONN, self.dsn
+            )
             log.debug("Created connection pool ({})".format(id(RawlConnection.pool)))
         else:
             log.debug("Reusing connection pool ({})".format(id(RawlConnection.pool)))
@@ -127,7 +137,7 @@ class RawlConnection(object):
         return True
 
     def get_conn(self):
-        log.info("Connecting to %s" % self.dsn)
+        log.debug("Retrieving connection from pool for %s" % self.dsn)
 
         conn = RawlConnection.pool.getconn()
         if conn.status not in OPEN_TRANSACTION_STATES:
@@ -297,7 +307,7 @@ class RawlBase(ABC):
 
         return query_string
 
-    def _execute(self, query, commit=False, working_columns=None):
+    def _execute(self, query, commit=True, working_columns=None, read_only=False):
         """
         Execute a query with provided parameters
 
@@ -330,12 +340,31 @@ class RawlBase(ABC):
         else:
             curs = conn.cursor()
 
-        try:
-            log.debug("Executing(%s): %s" % (query_id, query.as_string(curs)))
-        except Exception:
-            log.exception("LOGGING EXCEPTION LOL")
+        def _clean_up():
+            if not self._open_cursor:
+                log.debug("Closing cursor")
+                curs.close()
 
-        curs.execute(query)
+            if not self._open_transaction:
+                log.debug("put_conn({})".format(id(conn)))
+                self._connection_manager.put_conn(conn)
+
+        try:
+            query_string = query.as_string(curs)
+        except Exception:
+            query_string = ""
+
+        log.debug("Executing(%s): %s" % (query_id, query_string))
+
+        try:
+            curs.execute(query)
+        except Exception as err:
+            log.exception("Exception occurred when executing query: {}".format(query_string))
+
+            _clean_up()
+
+            # This still should be handled by the user of this lib
+            raise err
 
         log.debug("Executed")
 
@@ -368,11 +397,7 @@ class RawlBase(ABC):
                 rr = RawlResult(working_columns, row_dict)
                 result.append(rr)
 
-        if not self._open_cursor:
-            curs.close()
-
-        if not self._open_transaction:
-            self._connection_manager.put_conn(conn)
+        _clean_up()
 
         return result
 
@@ -402,12 +427,12 @@ class RawlBase(ABC):
         :commit:        Whether or not to commit the transaction after the query
         :returns:       Psycopg2 result
         """
-        commit = None
-        columns = None
-        if kwargs.get("commit") is not None:
-            commit = kwargs.pop("commit")
-        if kwargs.get("columns") is not None:
-            columns = kwargs.pop("columns")
+        commit = pop_or_none(kwargs, "commit")
+        columns = pop_or_none(kwargs, "columns")
+
+        if commit is None and self._open_transaction is not None:
+            commit = False
+
         query = self._assemble_simple(sql_string, *args, **kwargs)
         return self._execute(query, commit=commit, working_columns=columns)
 
@@ -420,13 +445,18 @@ class RawlBase(ABC):
         :*args:         Arguments to be passed for query parameters.
         :returns:       Psycopg2 result
         """
-        working_columns = None
-        if kwargs.get("columns") is not None:
-            working_columns = kwargs.pop("columns")
-        query = self._assemble_with_columns(sql_string, cols, *args, *kwargs)
-        return self._execute(query, working_columns=working_columns)
 
-    def insert_dict(self, value_dict, commit=False):
+        commit = pop_or_none(kwargs, "commit")
+        working_columns = pop_or_none(kwargs, "columns")
+
+        if commit is None and self._open_transaction is not None:
+            commit = False
+
+        query = self._assemble_with_columns(sql_string, cols, *args, *kwargs)
+
+        return self._execute(query, working_columns=working_columns, commit=commit)
+
+    def insert_dict(self, value_dict, commit=True):
         """
         Execute an INSERT statement using a python dict
 
@@ -531,9 +561,14 @@ class RawlBase(ABC):
         """
         if self._open_transaction:
             log.debug("rollback()")
+
             if self._open_cursor:
                 self._open_cursor.close()
+
             self._open_transaction.rollback()
+            self._connection_manager.put_conn(self._open_transaction)
+
+            self._open_cursor = None
             self._open_transaction = None
         else:
             log.warning("Cannot rollback, no open transaction")
@@ -544,9 +579,14 @@ class RawlBase(ABC):
         """
         if self._open_transaction:
             log.debug("commit()")
+
             if self._open_cursor:
                 self._open_cursor.close()
+
             self._open_transaction.commit()
+            self._connection_manager.put_conn(self._open_transaction)
+
+            self._open_cursor = None
             self._open_transaction = None
         else:
             log.warning("Cannot commit, no open transaction")
